# coding=UTF-8
# 定义参数列表
import argparse
parser = argparse.ArgumentParser(
    description='A offline-software package for bioluminescent proteins identification in batch. Linux and windows are suitable.')
    # 这里的描述会出现在 usage下方 表明这个程序的作用是什么
parser.add_argument("-t","--PredictType", required=True, type=str, default="G", help='Predict type for bioluminescent proteins identification, default=G.',
                    choices=["G","B","E","A"])
parser.add_argument("-i","--InputFile", required=True, default=None, help='Input data with fasta format suffixed with .txt, fasta or .fa.')
args = parser.parse_args()

def generate_finally_result_file(inSeqFile,PredictFile, finallyresultfile):
    seqfile = []
    f1 = open(inSeqFile)
    for eachline in f1:
        if eachline[0] == '>':
            seq_name = eachline.strip()#序列名称
        elif eachline[0] == '\n':
            continue
        seqfile.append(seq_name)
    f1.close()

    f2 = open(PredictFile,'r')
    lines = f2.readlines()
    g = open(finallyresultfile,'w')
    Noteline = 'QueryProtein'+'---'+'BLPs(Yes/No)' + '---' + 'PredictScores' + '\n'
    g.write(Noteline)
    for i in range(len(lines)):
        label = lines[i].strip().split('\t')[0] #标签
        proValue = lines[i].strip().split('\t')[1] #概率
        if label == '1':
            g.write(seqfile[i]+'---'+'Yes' + '---' +proValue+'\n')
        else:
            g.write(seqfile[i]+'---'+'No' + '---' +proValue+'\n')
    return finallyresultfile

def predict_model(modelfile,featrue_file,predict_result_file):
    #加载模型
    xgb_model = joblib.load(modelfile)
    #读取预测的特征文件
    test_dataset = pd.read_csv(featrue_file,header = 0)
    X_test = test_dataset.iloc[:,1:].values
    #模型预测
    preds = xgb_model.predict(X_test)
    dtest_predprob = xgb_model.predict_proba(X_test)[:,1] ##得到预测类别的概率
    dtest_predprob = np.around(dtest_predprob, decimals=3)
    #设置阈值, 输出一些评价指标
    y_pred = (preds >= 0.5)*1
    f = open(predict_result_file,'w')
    for i in range(len(y_pred)):
        f.write(str(y_pred[i])+'\t'+ str(dtest_predprob[i])+'\n')
    f.close()

import joblib
import pandas as pd
import numpy as np
import sys,os
from PythonScripts import FeatureExtractionForGeneral as FEG
from PythonScripts import FeatureExtractionForBacteria as FEB
from PythonScripts import FeatureExtractionForEukaryote as FEE
from PythonScripts import FeatureExtractionForArchaea as FEA

#predtype = sys.argv[1] #pred_type
#inSeqFile = sys.argv[2] #原始输入序列文件

predtype = args.PredictType
inSeqFile = args.InputFile

file_path = os.path.dirname(inSeqFile)
prefix_name = file_path + "/" + os.path.basename(inSeqFile).split('.')[0]

predict_result_file = prefix_name + "_predresult.txt" #预测结果
finallyresultfile = "./Output/" + "finalresult_" + os.path.basename(inSeqFile).split('.')[0] + ".txt"#最终在网页上展示的结果

def FeaExact(inSeqFile, predtype):
    if predtype == "G":
        FEG.main(inSeqFile)
    elif predtype == "B":
        FEB.main(inSeqFile)
    elif predtype == "E":
        FEE.main(inSeqFile)
    elif predtype == "A":
        FEA.main(inSeqFile)
       
if __name__ == '__main__':

    if predtype == "G":
        FeaExact(inSeqFile, "G")
        featrue_file = prefix_name + '_general.csv'
        modelfile = './Model/general_xgb.model'
        predict_model(modelfile,featrue_file,predict_result_file)
        finallyresultfile = generate_finally_result_file(inSeqFile, predict_result_file, finallyresultfile)
    
    elif predtype == "B":
        FeaExact(inSeqFile, "B")
        featrue_file = prefix_name + '_bacteria.csv'
        modelfile = './Model/bacteria_xgb.model'
        predict_model(modelfile,featrue_file,predict_result_file)
        finallyresultfile = generate_finally_result_file(inSeqFile, predict_result_file, finallyresultfile)
    
    elif predtype == "E":
        FeaExact(inSeqFile, "E")
        featrue_file = prefix_name + '_eukaryote.csv'
        modelfile = './Model/eukaryote_xgb.model'
        predict_model(modelfile,featrue_file,predict_result_file)
        finallyresultfile = generate_finally_result_file(inSeqFile, predict_result_file, finallyresultfile)
    
    elif predtype == "A":
        FeaExact(inSeqFile, "A")
        featrue_file = prefix_name + '_archaea.csv'
        modelfile = './Model/archaea_xgb.model'
        predict_model(modelfile,featrue_file,predict_result_file)
        finallyresultfile = generate_finally_result_file(inSeqFile, predict_result_file, finallyresultfile)
    
    #删除一些计算过程产生的特征文件等非必要文件，释放空间
    os.remove(featrue_file)
    os.remove(predict_result_file)
    print('Predict Type: ' + args.PredictType)
    print("The results are stored in a file: %s"%finallyresultfile)